# Generated by Django 4.2.10 on 2024-05-01 03:01

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0038_merge_20240425_0857"),
        ("database", "0038_merge_20240426_1640"),
    ]

    operations: list = []
