# Generated by Django 5.0.7 on 2024-09-12 05:43

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0060_merge_20240905_1828"),
    ]

    operations = [
        migrations.AlterField(
            model_name="texttoimagemodelconfig",
            name="model_type",
            field=models.CharField(
                choices=[("openai", "Openai"), ("stability-ai", "Stabilityai"), ("replicate", "Replicate")],
                default="openai",
                max_length=200,
            ),
        ),
    ]
